######################################################################
#
# File: b2sdk/_internal/account_info/__init__.py
#
# Copyright 2019 Backblaze Inc. All Rights Reserved.
#
# License https://www.backblaze.com/using_b2_code.html
#
######################################################################
from __future__ import annotations

from .in_memory import InMemoryAccountInfo

assert InMemoryAccountInfo
